@@ -13,7 +13,7 @@
 #' chains, that are only examined by few iterations (say less than 1000).
 #' @param pvalue A numeric between 0 and 1 denoting the threshold of p-values a variable in the imputation
 #' model should not exceed. If they do, they are excluded from the imputation model.
-#' @param rounding_degrees A numeric vector with the presumed rounding degrees.
+#' @param k An integer defining the allowed maximum of levels in a factor covariate.
 #' @return A list with 1. 'y_ret' the n x 1 data.frame with the original and imputed values.
 #' 2. 'Sol' the Gibbs-samples for the fixed effects parameters.
 #' 3. 'VCV' the Gibbs-samples for variance parameters.
@@ -25,7 +25,7 @@ imp_orderedcat_multi <- function(y_imp,
                       burnin = 5000,
                       thin = 20,
                       pvalue = 0.2,
-                      rounding_degrees = c(1, 10, 100, 1000)){
+                      k = Inf){
 
   # ----- checking input
   if(!is.factor(y_imp)){
@@ -33,12 +33,15 @@ imp_orderedcat_multi <- function(y_imp,
     y_imp <- as.factor(y_imp)
   }
 
+
+  # remove excessive variables
+  X_imp <- cleanup(X_imp, k = k)
   # standardise the covariates in X (which are numeric and no intercept)
-  X <- stand(X_imp, rounding_degrees = rounding_degrees)
+  X <- stand(X_imp)
 
   # -- standardise the covariates in Z (which are numeric and no intercept)
-  Z_imp <- cleanup(Z_imp)
-  Z <- stand(Z_imp, rounding_degrees = rounding_degrees)
+  Z_imp <- cleanup(Z_imp, k = k)
+  Z <- stand(Z_imp)
 
   #the missing indactor indicates, which values of y are missing.
   missind <- is.na(y_imp)
@@ -173,8 +176,12 @@ imp_orderedcat_multi <- function(y_imp,
   }
 
   #Remove highly correlated variables
+  #Less save alternative: get_type(X_model_matrix_1_sub)
+  tmptypes <- array(dim = ncol(X_model_matrix_1_sub))
+  for(j in 1:ncol(X_model_matrix_1_sub)){
 
-  tmptypes <- apply(X_model_matrix_1_sub, 2, get_type)
+    tmptypes[j] <- get_type(X_model_matrix_1_sub[, j])
+  }
   somethingcont <- tmptypes %in% c("cont", "binary", "roundedcont", "semicont")
   correlated <- NULL
   if(sum(somethingcont, na.rm = TRUE) >= 2){
