@@ -11,22 +11,22 @@
 #' @param X_imp A data.frame with the fixed effects variables.
 #' @param pvalue A numeric between 0 and 1 denoting the threshold of p-values a variable in the imputation
 #' model should not exceed. If they do, they are excluded from the imputation model.
-#' @param rounding_degrees A numeric vector with the presumed rounding degrees.
+#' @param k An integer defining the allowed maximum of levels in a factor covariate.
 #' @return A n x 1 data.frame with the original and imputed values.
 #' Note that this function won't return \code{interval} data as its purpose is to
 #' "break" the interval answers into precise answers.
 imp_interval <- function(y_imp,
                          X_imp,
                          pvalue = 0.2,
-                         rounding_degrees = c(1, 10, 100, 1000)){
+                         k = Inf){
 
 
   # ----------------------------- preparing the X data ------------------
   # remove excessive variables
-  X_imp <- cleanup(X_imp)
+  X_imp <- cleanup(X_imp, k = k)
 
   # standardize X
-  X <- stand(X_imp, rounding_degrees = rounding_degrees)
+  X <- stand(X_imp)
 
   # has to be numeric, so it must only consists of precise observations
   decomposed <- decompose_interval(interval = y_imp)
@@ -58,16 +58,17 @@ imp_interval <- function(y_imp,
   #will be exactly 0 and thus not significantly different from 0.
   #So in order to avoid this variable to be removed later in the code, we add +1.
   decomposed_stand <- (decomposed - y_mean)/y_sd + 1
-  y_precise_template_stand <- (y_precise_template - y_mean)/y_sd + 1
-
-  low_sample <- decomposed_stand[, "lower_general"]
-  up_sample  <- decomposed_stand[, "upper_general"]
-
-  y_precise_template <- msm::rtnorm(n = n, lower = low_sample,
-                                      upper = up_sample,
-                                      mean = y_precise_template_stand,
-                                      sd = 1)
 
+  #update y_precise_temple by draws from a truncated normal distribution
+  #which ensures, that the template values are in line with the bounds given in y_imp
+  y_precise_template <- msm::rtnorm(n = n,
+                                    lower = decomposed_stand[, "lower_general"],
+                                    upper = decomposed_stand[, "upper_general"],
+                                    mean = (y_precise_template - y_mean)/y_sd + 1,
+                                    sd = 1)
+  #For rare cases where it was not possible to draw a value from the truncated normal distribution,
+  #the sample_imp is used.
+  y_precise_template <- sample_imp(y_precise_template)[, 1]
   ph <- y_precise_template
 
 
@@ -164,17 +165,22 @@ imp_interval <- function(y_imp,
   keep <- (y_precise_template >= quants[1] & y_precise_template <= quants[2]) |
     is.na(y_precise_template)
 
-  m2 <- stats::optim(par = starting_values, negloglik2_intervalsonly,
-                     X = as.matrix(tmp_2_all[, xnames_1, drop = FALSE])[keep, , drop = FALSE],
-                     lower_bounds = decomposed_stand[, "lower_imprecise"][keep],
-                     upper_bounds = decomposed_stand[, "upper_imprecise"][keep],
-                     method = "BFGS",#alternative: "Nelder-Mead"
-                     control = list(maxit = 10000), hessian = TRUE)
+  m2 <- stats::nlm(f = negloglik2_intervalsonly, p = starting_values,
+                   parnames = names(starting_values),
+                   X = as.matrix(tmp_2_all[, xnames_1, drop = FALSE])[keep, , drop = FALSE],
+                   lower_bounds = decomposed_stand[, "lower_imprecise"][keep],
+                   upper_bounds = decomposed_stand[, "upper_imprecise"][keep],
+                   hessian = TRUE, gradtol = 1e-4, steptol = 1e-4)
 
 
-  par_ml2 <- m2$par
+  par_ml2 <- m2$estimate
+  names(par_ml2) <- names(starting_values)
   hess <- m2$hessian
 
+  if(m2$code > 2){
+    warning(paste("Likelihood-optimization for rounded continuous variable", colnames(y_imp), "failed."))
+  }
+
   # link about nearest covariance matrix:
   # http://quant.stackexchange.com/questions/2074/what-is-the-best-way-to-fix-a-covariance-matrix-that-is-not-positive-semi-defi
   # nearPD(hess)$mat
