@@ -3,21 +3,21 @@
 #' For example the income in surveys is often reported rounded by the respondents.
 #' See Drechsler, Kiesl and Speidel (2015) for more details.
 #' @param y_df A data.frame with the variable to impute.
-#' @param X A data.frame with the fixed effects variables.
+#' @param X_imp A data.frame with the fixed effects variables explaining y_df.
 #' @param PSI A data.frame with the variables explaining the latent rounding tendency G.
 #' @param pvalue A numeric between 0 and 1 denoting the threshold of p-values a variable in the imputation
 #' model should not exceed. If they do, they are excluded from the imputation model.
-#' @param rounding_degrees A numeric vector with the presumed rounding degrees.
-#' Or a list with model formulas for G, where each list element has the name of a rounded continuous variable.
-#' Such a list can be generated
+#' @param k An integer defining the allowed maximum of levels in a factor covariate.
+#' @param rounding_degrees A numeric vector with the presumed rounding degrees for Y.
 #' @references Joerg Drechsler, Hans Kiesl, Matthias Speidel (2015):
 #' "MI Double Feature: Multiple Imputation to Address Nonresponse and Rounding Errors in Income Questions".
 #' Austrian Journal of Statistics Vol. 44, No. 2, http://dx.doi.org/10.17713/ajs.v44i2.77
 #' @return A n x 1 data.frame with the original and imputed values.
 imp_roundedcont <- function(y_df,
-                            X,
+                            X_imp,
                             PSI,
                             pvalue = 0.2,
+                            k = Inf,
                             rounding_degrees = NULL){
 
   # ----------------------------- preparing the Y data ------------------
@@ -26,22 +26,17 @@ imp_roundedcont <- function(y_df,
     y <- as.interval(y)
   }
 
+  #currently the method does not consider 0 (no rounding)
+  no_rounding_at_all <- y %% 1 != 0
+  y <- round(y)
   n <- length(y)
 
   # ----------------------------- preparing the X data ------------------
   # remove excessive variables
-  X1 <- cleanup(X)
+  X_imp <- cleanup(X_imp, k = k)
 
   # standardize X
-  X <- stand(X1, rounding_degrees = rounding_degrees)
-
-  #the intercept variable shall be the first variable. Reason: later the intercept variable
-  #in the model based on this data set is excluded from optimization. Reason: the value of the
-  #intercept is exactly 1 if y and x are standardized.
-  tmp <- which(unlist(lapply(X, get_type)) == "intercept")
-
-  X <- X[, c(tmp, (1:ncol(X))[-tmp])]
-
+  X <- stand(X_imp)
 
   #If no default rounding_degrees were given, suggest_rounding_degrees suggets them
   if(is.null(rounding_degrees)){
@@ -55,7 +50,7 @@ imp_roundedcont <- function(y_df,
 
   #The imputation model of missing values is Y ~ X.
   #In order to get a full model matrix, we need two things
-  #1. A place holder ph with an precice structure
+  #1. A place holder ph with a precice structure
   #(meaning that ph is not of class interval. Nevertheless the elements in ph
   #can be an aggregate of imprecise observations (e.g. the mean of lower and upper bound))
   #2. The place holder ph must not contain any NAs, NaNs or Infs.
@@ -82,10 +77,9 @@ imp_roundedcont <- function(y_df,
   mean_of_y_precise <- mean(decomposed_y[, "precise"], na.rm = TRUE)
   sd_of_y_precise <- stats::sd(decomposed_y[, "precise"], na.rm = TRUE)
 
+  # Standardise all observations:
   # We intentionally add + 1 because otherwise with the standardized x,
-  # the intercept in the regression y ~ x can be exactly 0
-
-  # standardise all observations
+  # the intercept in the regression y ~ x can be exactly 0.
   y_stand <- (y - mean_of_y_precise)/sd_of_y_precise + 1
   y_precise_template <- sample_imp(center.interval(y, inf2NA = TRUE))[, 1]
   y_precise_template_stand <- (y_precise_template - mean_of_y_precise)/sd_of_y_precise + 1
@@ -103,19 +97,19 @@ imp_roundedcont <- function(y_df,
 
   ph <- y_precise_template
 
-
-  tmp_0_all <- data.frame(target = ph, X)
+  # prepare the estimation of Y ~ X
+  Y_X_0_all <- data.frame(target = ph, X)
   xnames_1 <- colnames(X)
 
-  tmp_formula <- paste("target~ 0 + ", paste(xnames_1, collapse = "+"), sep = "")
-  reg_1_all <- stats::lm(stats::formula(tmp_formula), data = tmp_0_all)
+  Y_X_formula <- paste("target~ 0 + ", paste(xnames_1, collapse = "+"), sep = "")
+  reg_Y_X_1_all <- stats::lm(stats::formula(Y_X_formula), data = Y_X_0_all)
 
-  X_model_matrix_1_all <- stats::model.matrix(reg_1_all)
+  X_model_matrix_1_all <- stats::model.matrix(reg_Y_X_1_all)
   xnames_1 <- paste("X", 1:ncol(X_model_matrix_1_all), sep = "")
   colnames(X_model_matrix_1_all) <- xnames_1
 
-  tmp_0_all <- data.frame(target = ph)
-  tmp_0_all[, xnames_1] <- X_model_matrix_1_all
+  Y_X_0_all <- data.frame(target = ph)
+  Y_X_0_all[, xnames_1] <- X_model_matrix_1_all
 
   #From this initial model matrix X_model_matrix_1_all
   #now step by step irrelavant variables are removed.
@@ -135,27 +129,27 @@ imp_roundedcont <- function(y_df,
   # (e.g. due to colinearity) on y.
   # tmp_1 shall include the covariates (like X_model_matrix) and additionally the target variable
   ph_sub <- ph[use_indicator]
-  tmp_1_sub <- data.frame(target = ph_sub)
+  X_Y_1_sub <- data.frame(target = ph_sub)
   xnames_1 <- colnames(X_model_matrix_1_sub)
-  tmp_1_sub[, xnames_1] <- X_model_matrix_1_sub
+  X_Y_1_sub[, xnames_1] <- X_model_matrix_1_sub
 
-  tmp_formula <- paste("target~ 0 + ", paste(xnames_1, collapse = "+"), sep = "")
-  reg_1_sub <- stats::lm(stats::formula(tmp_formula) , data = tmp_1_sub)
+  Y_X_formula <- paste("target~ 0 + ", paste(xnames_1, collapse = "+"), sep = "")
+  reg_Y_X_1_sub <- stats::lm(stats::formula(Y_X_formula) , data = X_Y_1_sub)
 
   #remove unneeded variables
-  X_model_matrix_1_sub <- X_model_matrix_1_sub[, !is.na(stats::coefficients(reg_1_sub)),
+  X_model_matrix_1_sub <- X_model_matrix_1_sub[, !is.na(stats::coefficients(reg_Y_X_1_sub)),
                                                drop = FALSE]
 
   # Remove insignificant variables from the imputation model
   check <- TRUE
   while(check){
-    tmp_1_sub <- data.frame(target = ph_sub)
+    X_Y_1_sub <- data.frame(target = ph_sub)
     xnames_1 <- colnames(X_model_matrix_1_sub)
-    tmp_1_sub[, xnames_1] <- X_model_matrix_1_sub
-    tmp_formula <- paste("target~ 0 + ", paste(xnames_1, collapse = "+"), sep = "")
-    reg_1_sub <- stats::lm(stats::formula(tmp_formula), data = tmp_1_sub)
+    X_Y_1_sub[, xnames_1] <- X_model_matrix_1_sub
+    Y_X_formula <- paste("target~ 0 + ", paste(xnames_1, collapse = "+"), sep = "")
+    reg_Y_X_1_sub <- stats::lm(stats::formula(Y_X_formula), data = X_Y_1_sub)
 
-    pvalues <- summary(reg_1_sub)$coefficients[, 4]
+    pvalues <- summary(reg_Y_X_1_sub)$coefficients[, 4]
     insignificant_variables <- which(pvalues > pvalue)
     most_insignificant <- insignificant_variables[which.max(pvalues[insignificant_variables])]
 
@@ -163,7 +157,7 @@ imp_roundedcont <- function(y_df,
       check <- FALSE
     }else{
       #.. drop the insignificant variable from the model.matrix, but only if at least 1 variable remains
-      tmp_MM <- stats::model.matrix(reg_1_sub)[, -most_insignificant, drop = FALSE]
+      tmp_MM <- stats::model.matrix(reg_Y_X_1_sub)[, -most_insignificant, drop = FALSE]
       if(ncol(tmp_MM) == 0){
         check <- FALSE
       }else{
@@ -172,39 +166,53 @@ imp_roundedcont <- function(y_df,
     }
   }
 
-  tmp_2_all <- tmp_0_all[, colnames(tmp_1_sub), drop = FALSE]
+  Y_X_2_all <- Y_X_0_all[, colnames(X_Y_1_sub), drop = FALSE]
 
-  # --preparing the ml estimation
-  # -define rounding intervals
+
+  betastart <- as.vector(reg_Y_X_1_sub$coef)
+  sigmastart <- stats::sigma(reg_Y_X_1_sub)
+  if(is.na(sigmastart)) sigmastart <- 1
+
+  ##### Preparation of rounding degrees start model #######
 
   half_interval_length <- rounding_degrees/2
 
   # Determine the rounding degrees of the precise observations
 
-  rounding_categories_indicator <- array(dim = c(sum(indicator_precise),
-                                                 length(rounding_degrees)))
-  for(i in 1:ncol(rounding_categories_indicator)){
-    rounding_categories_indicator[, i] <- decomposed_y[indicator_precise, "precise"] %% rounding_degrees[i] == 0
+  rounding_categories_indicator <- array(0, dim = sum(indicator_precise))
+
+
+  for(i in 1:length(rounding_degrees)){
+    rounding_categories_indicator <- ifelse(decomposed_y[indicator_precise, "precise"] %% rounding_degrees[i] == 0,
+                                            i, rounding_categories_indicator)
   }
 
-  g <- factor(rowSums(rounding_categories_indicator), ordered = TRUE)
+  g <- factor(rounding_categories_indicator, ordered = TRUE)
 
-  #preparing the PSI data (the variables explaining the rounding tendency G)
-  # From interval variables, only the precise part can be used.
-  vars_being_intervals <- apply(PSI, 2, is_interval)
-  vars_being_intervals[names(vars_being_intervals) == colnames(y_df)[1]] <- FALSE
-  PSI <- PSI[, !vars_being_intervals, drop = FALSE]
+  ##### preparing the PSI data (the variables explaining the rounding tendency G) ####
+  #check if y is part of PSI
+  is_y_in_PSI <- colnames(y_df)[1] %in% colnames(PSI)
+  # reason: it is a special covariate in PSI.
+  # It has a separat coefficient and is the only variable allowed being an interval variable.
+
+  PSIvars_being_intervals <- apply(PSI, 2, is_interval)
+  PSI <- PSI[, !PSIvars_being_intervals, drop = FALSE]
 
-  # The target variable in PSI has be to present precisely
-  PSI[, colnames(y_df)[1]] <- decompose_interval(PSI[, colnames(y_df)[1]])[, "precise"]
   # This dataset must not include any NA, except y can include missing values, as
   # the rounding tendency is only calculated based on precise observations
-  vars_having_NAs <- apply(PSI, 2, function(x) any(is.na(x)))
-  vars_having_NAs[names(vars_having_NAs) == colnames(y_df)[1]] <- FALSE
-  PSI <- PSI[, !vars_having_NAs, drop = FALSE]
-  PSI <- stand(PSI, rounding_degrees = NULL)
+  PSIvars_having_NAs <- apply(PSI, 2, function(x) any(is.na(x)))
+  PSI <- PSI[, !PSIvars_having_NAs, drop = FALSE]
+  #if y was a part of PSI, its precise part is included
+  if(is_y_in_PSI){
+    PSI[, colnames(y_df)[1]] <- decomposed_y[, "precise", drop = FALSE]
+  }
+  #standardize PSI
+  PSI <- stand(PSI)
+
   # Define a matrix for the model G ~ PSI
-  df_for_g <- data.frame(target = g, PSI[indicator_precise, , drop = FALSE])
+  df_for_g_sub <- data.frame(target = g, PSI[indicator_precise, , drop = FALSE])
+  #note: even if Y is not part of PSI, the conditioning on precise y is needed:
+  #imprecise Y (like "2000;3000") don't have a rounding degree.
 
   #####maximum likelihood estimation using starting values
   ####estimation of the parameters
@@ -219,14 +227,14 @@ imp_roundedcont <- function(y_df,
       #(See ?polr)
       #so we add one in the formula and exclude the constant variable in the data.frame
       #before hand.
-      constant_variables <- apply(df_for_g, 2, function(x) length(unique(x)) == 1)
-      df_for_g_2 <- df_for_g[, !constant_variables, drop = FALSE]
-      if(ncol(df_for_g_2) == 1){
+      constant_variables <- apply(df_for_g_sub, 2, function(x) length(unique(x)) == 1)
+      df_for_g_sub2 <- df_for_g_sub[, !constant_variables, drop = FALSE]
+      if(ncol(df_for_g_sub2) == 1){
         probitstart <- VGAM::vglm("target ~ 0 + .",
-                                  data = df_for_g,
+                                  data = df_for_g_sub,
                                   family = VGAM::cumulative(parallel = TRUE))
       }else{
-        probitstart <- VGAM::vglm("target ~ 1 + .", data = df_for_g_2,
+        probitstart <- VGAM::vglm("target ~ 1 + .", data = df_for_g_sub2,
                                   family = VGAM::cumulative(parallel = TRUE))
 
       }
@@ -246,14 +254,14 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
 Consider specifying the variable to be continuous via list_of_types (see ?hmi).\n")
       cat("Here is the original warning message:\n")
       cat(as.character(cond))
-      constant_variables <- apply(df_for_g, 2, function(x) length(unique(x)) == 1)
-      df_for_g_2 <- df_for_g[, !constant_variables, drop = FALSE]
-      if(ncol(df_for_g_2) == 1){
+      constant_variables <- apply(df_for_g_sub, 2, function(x) length(unique(x)) == 1)
+      df_for_g_sub2 <- df_for_g_sub[, !constant_variables, drop = FALSE]
+      if(ncol(df_for_g_sub2) == 1){
         probitstart <- VGAM::vglm("target ~ 0 + .",
-                                  data = df_for_g,
+                                  data = df_for_g_sub,
                                   family = VGAM::cumulative(parallel = TRUE))
       }else{
-        probitstart <- VGAM::vglm("target ~ 1 + .", data = df_for_g_2, family = VGAM::cumulative(parallel = TRUE))
+        probitstart <- VGAM::vglm("target ~ 1 + .", data = df_for_g_sub2, family = VGAM::cumulative(parallel = TRUE))
 
       }
       return(probitstart)
@@ -264,45 +272,42 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
   )
 
 
-  PSI_as_MM <- stats::model.matrix(probitstart)[1:nrow(stats::model.matrix(probitstart)),
+  PSI_as_MM_sub <- stats::model.matrix(probitstart)[1:nrow(stats::model.matrix(probitstart)),
                                                 1:ncol(stats::model.matrix(probitstart))]
   #remove the intercept variables
-  PSI_as_MM <- PSI_as_MM[, -grep("(Intercept)", colnames(PSI_as_MM)), drop = FALSE]
-  #remove target variable
-  PSI_as_MM <- PSI_as_MM[, -grep(colnames(y_df)[1], colnames(PSI_as_MM)), drop = FALSE]
+  PSI_as_MM_sub <- PSI_as_MM_sub[, -grep("(Intercept)", colnames(PSI_as_MM_sub)), drop = FALSE]
+  #old: remove target variable
+  #PSI_as_MM <- PSI_as_MM[, -grep(colnames(y_df)[1], colnames(PSI_as_MM)), drop = FALSE]
   #Individuals are containted multiple times in the model.matrices returned by VGAM,
-  #dependent on how many categories there are. The presumed purpose is to genereta (K-1) intercept dummies
+  #dependent on how many categories there are. The presumed purpose is to generet (K-1) intercept dummies
 
-  PSI_as_MM <- PSI_as_MM[seq(1, nrow(PSI_as_MM), by = length(unique(g))-1), , drop = FALSE]
+  PSI_as_MM_sub <- PSI_as_MM_sub[seq(1, nrow(PSI_as_MM_sub), by = length(unique(g))-1), , drop = FALSE]
 
   tmp <- attr(probitstart, "coefficients")
   kstart <- as.vector(tmp[grep("(Intercept)", names(tmp))])
-    # the tresholds (in the summary labeled "Intercepts")
+  # the tresholds (in the summary labeled "Intercepts")
   #explaining the tresholds for the example of rounding degrees 1, 10, 100 and 1000:
   #0 (rounding degree 1), 0|1 (reounding degree 10),  1|2 (100),  2|3 (1000)
 
-  # it might be more practical to run the model
-  #only based on the observed data, but this could cause some covariates in betastart2 to be dropped
-  betastart <- as.vector(reg_1_sub$coef)
-  sigmastart <- stats::sigma(reg_1_sub)
 
   gammastart <- tmp[-grep("(Intercept)", names(tmp))]
-  gamma1start <- gammastart[names(gammastart) == colnames(y_df)[1]]
-  gamma1name <- "gamma1"
 
-  vars_in_psi <- names(gammastart)[names(gammastart) != colnames(y_df)[1]]
-  gammastart_without_y <- gammastart[vars_in_psi]
-  # It can happen, that the user does not want y to be an explanatory variable.
-  # In this case, gamma1 has to be set to 0 and shall not be part of the optimization
-  if(length(gamma1start) == 0){
-    gamma1start <- NULL
-    gamma1name <- NULL
+
+  gamma1start <- NULL
+  gamma1name <- NULL
+  if(is_y_in_PSI){
+    gamma1start <- gammastart[names(gammastart) == colnames(y_df)[1]]
+    gamma1name <- "gamma1"
   }
 
-  gamma_without_y_name <- paste("coef_g_on_psi", 1:length(gammastart_without_y), sep = "")
+    # If Y is part of PSI, its coefficient gamma1 has to be treated seperately
+  # (For example, gamma1 is part of the covariance matrix in equation 3 in Drechsler, Kiesl, Speidel, 2015)
+  gammastart_without_y <- gammastart[names(gammastart) != colnames(y_df)[1]]
+  gammastart_without_y_name <- paste("coef_g_on_psi", 1:length(gammastart_without_y), sep = "")
+  #If Y would be the only variable, set gamma_without_y_name NULL
   if(length(gammastart_without_y) == 0){
     gammastart_without_y <- NULL
-    gamma_without_y_name <- NULL
+    gammastart_without_y_name <- NULL
   }
 
   #####maximum likelihood estimation using the starting values
@@ -319,7 +324,7 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
     paste("coef_y_on_x", 1:length(betastart), sep = "")
 
   names(starting_values)[length(kstart) + length(betastart) +
-                          1:length(gammastart)] <- c(gamma1name, gamma_without_y_name)
+                          1:length(gammastart)] <- c(gamma1name, gammastart_without_y_name)
 
   names(starting_values)[length(starting_values)] <- "sigma"
   ###exclude obs below (above) the 0.5% (99.5%) income quantile before maximizing
@@ -332,26 +337,29 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
   indicator_outliers <- (decomposed_y_stand[indicator_precise, "precise"] < quants[1] |
                          decomposed_y_stand[indicator_precise, "precise"] > quants[2])
 
-  #The intercept in beta is not part of the maximazition, as due to standardizations of y and x,
-  #its value is exactly 1#[-(length(kstart) + 1)].
-  m2 <- stats::optim(par = starting_values, negloglik,
-                     X_in_negloglik = tmp_2_all[ , xnames_1, drop = FALSE],
-                     PSI_in_negloglik = PSI_as_MM,
-                     vars_in_psi = vars_in_psi,
-                     y_precise_stand = decomposed_y_stand[indicator_precise, "precise"],
-                     lower_bounds = decomposed_y_stand[indicator_imprecise, 2],
-                     upper_bounds = decomposed_y_stand[indicator_imprecise, 3],
-                     my_g = as.numeric(as.character(g)),
-                     sd_of_y_precise = sd_of_y_precise,
-                     rounding_degrees = rounding_degrees,
-                     indicator_precise = indicator_precise,
-                     indicator_imprecise = indicator_imprecise,
-                     indicator_outliers = indicator_outliers,
-                     method = "Nelder-Mead",#"BFGS",
-                     control = list(maxit = 10000), hessian = TRUE)
+  PSI_in_negloglik <- PSI_as_MM_sub[, colnames(PSI_as_MM_sub) != colnames(y_df)[1], drop = FALSE]
 
-  par_ml2 <- m2$par
+  m2 <- stats::nlm(f = negloglik, p = starting_values,
+                   parnames = names(starting_values),
+                   X_in_negloglik = Y_X_2_all[ , xnames_1, drop = FALSE],
+                   PSI_in_negloglik = PSI_in_negloglik,
+                   y_precise_stand = decomposed_y_stand[indicator_precise, "precise"],
+                   lower_bounds = decomposed_y_stand[indicator_imprecise, 2],
+                   upper_bounds = decomposed_y_stand[indicator_imprecise, 3],
+                   my_g = as.numeric(as.character(g)),
+                   sd_of_y_precise = sd_of_y_precise,
+                   rounding_degrees = rounding_degrees,
+                   indicator_precise = indicator_precise,
+                   indicator_imprecise = indicator_imprecise,
+                   indicator_outliers = indicator_outliers,
+                   hessian = TRUE, gradtol = 1e-4, steptol = 1e-4)
+
+  par_ml2 <- m2$estimate
+  names(par_ml2) <- names(starting_values)
   hess <- m2$hessian
+  if(m2$code > 2){
+    warning(paste("Likelihood-optimization for rounded continuous variable", colnames(y_df), "failed."))
+  }
 
   # link about nearest covariance matrix:
   # http://quant.stackexchange.com/questions/2074/what-is-the-best-way-to-fix-a-covariance-matrix-that-is-not-positive-semi-defi
@@ -365,8 +373,6 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
     error = function(cond) {
       cat("Hessian matrix couldn't be inverted (in the imputation function of the rounded continuous variable).
               Still, you should get a result, but which needs special attention.\n")
-      cat("Here is the original error message:\n")
-      cat(as.character(cond))
 
       tmp <- diag(ncol(hess))
       diag(tmp) <- abs(par_ml2)/100
@@ -414,9 +420,15 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
   gamma1_hat <- pars[grep("^gamma1", colnames(pars))]
   gamma_hat <- matrix(pars[grep("^coef_g_on_psi", colnames(pars))], ncol = 1)
   sigma_hat <- pars[grep("^sigma", colnames(pars))]
-  mu_g <- gamma1_hat * (as.matrix(tmp_2_all[, xnames_1, drop = FALSE]) %*% beta_hat) +
-    as.matrix(PSI[, vars_in_psi, drop = FALSE], nrow = nrow(PSI)) %*% gamma_hat
-  mu_y <- as.matrix(tmp_2_all[, xnames_1, drop = FALSE]) %*% beta_hat
+
+  #For the potentially ROUNDED observations, the potential rounding tendency has to be modeled
+  mu_g <- gamma1_hat * (as.matrix(Y_X_2_all[indicator_precise, xnames_1, drop = FALSE]) %*% beta_hat) +
+    PSI_in_negloglik %*% gamma_hat
+
+  #For the ROUNDED, MISSING and INTERVAL observations, y has to be modeled.
+  mu_y <- as.matrix(Y_X_2_all[, xnames_1, drop = FALSE]) %*% beta_hat
+
+  #so mu_g and mu_y generally differ in length!
 
   #The covariance matrix from equation (3)
   Sigma <- matrix(c(1 + gamma1_hat^2 * sigma_hat^2,
@@ -465,7 +477,7 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
 
   g_upper <- bounds_for_g_hat[as.numeric(as.character(g))[actually_rounded] + 1]
 
-  elements <- cbind(-Inf, mu_g[indicator_precise, 1][actually_rounded], g_upper,
+  elements <- cbind(-Inf, mu_g[actually_rounded], g_upper,
                     y_lower, mu_y[indicator_precise, 1][actually_rounded], y_upper)
 
   # Note: we set g_lower to -Inf because we state that a value of 1500 is not necessarily
@@ -478,15 +490,22 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
   rounded <- rep(TRUE, sum(actually_rounded))
 
   counter <- 0
-  while(any(rounded)){
+  while(any(rounded) & counter <= 1000){
     counter <- counter + 1
 
+    if(counter == 1000){
+      print(paste("Imputation of rounded continuous variable", colnames(y_df)[1], "failed."))
+    }
     ###draw values for g and y from a truncated multivariate normal
     ###drawn y must be between y_lower and y_upper
     ###drawn g must be between g_lower and g_upper
-    if(counter >= 50){
+    #If the model has some problems finding a suitable rounding degree,
+    #the expected mean of g is increased to the threshold seperating the highest and second highest
+    # category
+    if(counter >= 100){
       elements[, 2] <- bounds_for_g_hat[length(bounds_for_g_hat)-1]
     }
+
     mytry <- t(apply(elements[rounded, , drop = FALSE],
                     1, sampler, Sigma))
 
@@ -502,7 +521,7 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
     #So, if for individual i no valid value for g or y could be sampled,
     # it could either be because mu_g[i] lies outisde of the interval
     #[g_lower[i];g_upper[i]] or because mu_y[i] outside of y_lower[i];y_upper[i]].
-    # We then check whether it is mu_g or mu_y, that loes outside its interval
+    # We then check whether it is mu_g or mu_y, that lies outside its interval
     #and then replace the corresponding mean
     #by a uniform sample between the lower and the upper bound.
 
@@ -512,8 +531,17 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
     # For y, we use a uniform sample between the highest and lowest possible
     #bounds of y.
     problematic_draws <- is.na(mytry[, 1])
+
     problematic_elements <- elements[problematic_draws, , drop = FALSE]
 
+    #If the model has even further deficiancies, all remaining elements are considered problematic
+    if(counter >= 200){
+      toosmall_ys <- elements[, 5] < elements[, 4]
+      toolarge_ys <- elements[, 5] > elements[, 6]
+      elements[toosmall_ys, 5] <- elements[toosmall_ys, 4]
+      elements[toolarge_ys, 5] <- elements[toolarge_ys, 6]
+    }
+
     # check if there are problematic means of g. This is the case if the mean is outside
     # the interval for a possible g.
     toosmall_gs <- problematic_elements[, 2] < problematic_elements[, 1]
@@ -554,8 +582,11 @@ Consider specifying the variable to be continuous via list_of_types (see ?hmi).\
     domatch <- floor(imp_precise_temp[, 1]/rounding_degrees[round_int] + 0.5) * rounding_degrees[round_int] ==
                           decomposed_y[indicator_precise, "precise"][actually_rounded][rounded]
     rounded[rounded][domatch] <- FALSE
+
   }
 
+  #restore the original values for those observations with no rounding at all (rounding degree 0).
+  imp_tmp[no_rounding_at_all] <- y_df[no_rounding_at_all, 1]
   y_ret <- data.frame(y_ret = imp_tmp)
 
   return(y_ret)
