@@ -11,6 +11,7 @@
 #' @param model_formula A \code{\link[stats]{formula}} used for the analysis model.
 #' Currently the package is designed to handle formula used in
 #' \code{lm}, \code{glm} and \code{lmer}.
+#' @param family A family object supported by \code{glm} (resp. \code{glmer}). See \code{family}.
 #' @param additional_variables A character with names of variables (separated by "+", like "x8+x9")
 #' that should be included in the imputation model as fixed effects variables,
 #' but not in the analysis model.
@@ -23,23 +24,20 @@
 #' @param nitt An integer defining number of MCMC iterations (see \code{MCMCglmm}).
 #' @param burnin An integer for the desired number of
 #' Gibbs samples that shall be regarded as burnin.
-#' @param thin An integer to set the thinning interval range. If thin = 1,
-#' every iteration of the Gibbs-sampling chain will be kept. For highly autocorrelated
-#' chains, that are only examined by few iterations (say less than 1000),
-#' the \code{geweke.diag} might fail to detect convergence. In such cases it is
-#' essential to look a chain free from autocorelation.
 #' @param pvalue A numeric between 0 and 1 denoting the threshold of p-values a variable in the imputation
 #' model should not exceed. If they do, they are excluded from the imputation model.
 #' @param mn An integer defining the minimum number of individuals per cluster.
-#' @param spike A numeric value saying which value in the semi-continuous data might is the spike.
+#' @param k An integer defining the allowed maximum of levels in a factor covariate.
+#' @param spike A numeric value saying which value in the semi-continuous data might be the spike.
 #' Or a list with with such values and names identical to the variables with spikes
 #' (see \code{list_of_spikes_maker} for details.) In versions earlier to 0.9.0 it was called \code{heap}.
 #' @param heap Use spike instead. \code{heap} is only included due to backwards compatibility
 #' and will be removed with version 1.0.0
-#' @param rounding_degrees A numeric vector with the presumed rounding degrees. Or a list with rounding degrees,
-#' where each list element has the name of a rounded continuous variable. Such a list can be generated
-#' using \code{list_of_rounding_degrees_maker(data)}. Note: it is presupposed that the rounding degrees include 1
-#' meaning that there is a positive probability that e.g. 3500 was only rounded to the nearest integer
+#' @param rounding_degrees A numeric vector with the presumed rounding degrees of rounded variables.
+#' Or a list with rounding degrees,  where each list element has the name of a rounded continuous variable.
+#' Such a list can be generated using \code{list_of_rounding_degrees_maker(data)}.
+#' Note: it is presupposed that the rounding degrees include 1 meaning that there
+#' is a positive probability that e.g. 3500 was only rounded to the nearest integer
 #' (and not rounded to the nearest multiple of 100 or 500).
 #' @param rounding_formula A formula with the model formula for the latent rounding tendency G.
 #' Or a list with model formulas for G, where each list element has the name of a rounded continuous variable.
@@ -60,50 +58,52 @@
 #' @return The function returns a \code{mids} object. See \code{mice} for further information.
 #' @examples
 #' \dontrun{
-#' my.formula <- Reaction ~ Days + (1 + Days|Subject)
-#' my_analysis <- function(complete_data){
-#'  # In this list, you can write all the parameters you are interested in.
-#'  # Those will be averaged.
-#'  # So make sure that averaging makes sense and that you only put in single numeric values.
-#'  parameters_of_interest <- list()
+#'  require("lme4")
+#'  require("mice")
+#'  data(Gcsemv, package = "hmi")
 #'
-#'  # ---- write in the following lines, what you are interetest in to do with your complete_data
-#'  # the following lines are an example where the analyst is interested in the fixed intercept
-#'  # and fixed slope and the random intercepts variance,
-#'  # the random slopes variance and their covariance
-#'  my_model <- lmer(my.formula, data = complete_data)
+#'  model_formula <- written ~ 1 + gender + coursework + (1 + gender|school)
 #'
-#'  parameters_of_interest[[1]] <- fixef(my_model)[1]
-#'  parameters_of_interest[[2]] <- fixef(my_model)[2]
-#'  parameters_of_interest[[3]] <- VarCorr(my_model)[[1]][1, 1]
-#'  parameters_of_interest[[4]] <- VarCorr(my_model)[[1]][1, 2]
-#'  parameters_of_interest[[5]] <- VarCorr(my_model)[[1]][2, 2]
-#'  names(parameters_of_interest) <- c("beta_intercept", "beta_Days", "sigma0", "sigma01", "sigma1")
+#'  set.seed(123)
+#'  dat_imputed <- hmi(data = Gcsemv, model_formula = model_formula, M = 2, maxit = 2)
 #'
-#'  # ---- do change this function below this line.
-#'  return(parameters_of_interest)
+#'  my_analysis <- function(complete_data){
+#'   # In this list, you can write all the parameters you are interested in.
+#'   # Those will be averaged.
+#'   # So make sure that averaging makes sense and that you only put in single numeric values.
+#'   parameters_of_interest <- list()
+#'
+#'   # ---- write in the following lines, what you are interetest in to do with your complete_data
+#'   # the following lines are an example where the analyst is interested in the fixed intercept
+#'   # and fixed slope and the random intercepts variance,
+#'   # the random slopes variance and their covariance
+#'   my_model <- lmer(model_formula, data = complete_data)
+#'
+#'   parameters_of_interest[[1]] <- fixef(my_model)
+#'   parameters_of_interest[[2]] <- lme4::VarCorr(my_model)[[1]][,]
+#'   ret <- unlist(parameters_of_interest)# This line is essential if the elements of interest
+#'   #should be labeled in the following line.
+#'   names(ret) <-
+#'     c("beta_intercept", "beta_gender", "beta_coursework", "sigma0", "sigma01", "sigma10", "sigma1")
+#'
+#'   return(ret)
 #' }
-#' require("lme4")
-#' require("mice")
-#' data(sleepstudy, package = "lme4")
-#' test <- sleepstudy
-#' test[sample(1:nrow(test), size = 20), "Reaction"] <- NA
-#' hmi_imp <- hmi(data = test, model_formula = my.formula, M = 5, maxit = 1)
-#' hmi_pool(mids = hmi_imp, analysis_function = my_analysis)
+#' hmi_pool(mids = dat_imputed, analysis_function = my_analysis)
 #' #if you are interested in fixed effects only, consider pool from mice:
-#' pool(with(data = hmi_imp, expr = lmer(Reaction ~ Days + (1 + Days | Subject))))
+#' pool(with(data = dat_imputed, expr = lmer(written ~ 1 + gender + coursework + (1 + gender|school))))
 #' }
 #' @export
 hmi <- function(data,
                 model_formula = NULL,
+                family = stats::gaussian,
                 additional_variables = NULL,
                 M = 5,
                 maxit = NULL,
-                nitt = 25000,
-                burnin = 5000,
-                thin = 20,
+                nitt = 12000,
+                burnin = 2000,
                 pvalue = 1,
                 mn = 1,
+                k = NULL,
                 spike = NULL,
                 heap = NULL,
                 rounding_degrees = NULL,
@@ -118,20 +118,30 @@ hmi <- function(data,
     if(is.null(spike)) spike <- heap
   }
 
-  if(is.null(spike)){
-    heap <- list_of_spikes_maker(data)
-  }else{
-    heap <- spike
+  #restore the original value of k. By this later it is possible whether a value of k = Inf,
+  #was given by the user, or set as default by hmi.
+  k_org <- k
+  if(is.null(k)){
+    k <- Inf
   }
-  if(is.null(heap)) heap <- 0
+  if(is.null(spike)){
+    spike <- list_of_spikes_maker(data)
+  }
+  if(is.null(spike)) spike <- 0
 
   if(is.null(list_of_types)){
-    tmp_list_of_types <- list_of_types_maker(data, rounding_degrees = rounding_degrees)
+    tmp_list_of_types <- list_of_types_maker(data, spike = spike, rounding_degrees = rounding_degrees)
   }else{
     if(!is.list(list_of_types)) stop("We need list_of_types to be a list.")
-    if(!isTRUE(all.equal(sort(names(list_of_types)), sort(colnames(data))))){
-      stop("At least one variable in list_of_types isn't found in data or vica versa.")
+
+    if(any(!names(list_of_types) %in% colnames(data))){
+      stop("At least one variable in list_of_types isn't found in data.")
     }
+    #Currently we allow list_of_types to specify only a subset of variables found in data.
+    #For a strict 1:1 relation, the following code can be used.
+    #if(!isTRUE(all.equal(sort(names(list_of_types)), sort(colnames(data))))){
+    #  stop("At least one variable in list_of_types isn't found in data or vica versa.")
+    #}
 
     if(any(lapply(list_of_types, class) != "character")){
       stop("We need the elements of list_of_types to be of class character.")
@@ -144,6 +154,20 @@ hmi <- function(data,
     tmp_list_of_types <- list_of_types
   }
 
+  #Check for implicit specifications for a variable to be semicont or roundedcont
+  if(is.list(spike)){
+    for(varindex in colnames(data)){
+      if(is.null(list_of_types[[varindex]]) & !is.null(spike[[varindex]])){
+        tmp_list_of_types[[varindex]] <- "semicont"
+      }
+
+      if(is.null(list_of_types[[varindex]]) & !is.null(rounding_degrees[[varindex]])){
+        tmp_list_of_types[[varindex]] <- "roundedcont"
+      }
+    }
+  }
+
+
   my_data <- data
 
   if(is.matrix(my_data)) stop("We need your data in a data.frame (and not a matrix).")
@@ -188,12 +212,12 @@ hmi <- function(data,
   # Set a default value of maxit.
   # This default value should be 1 in case of only one variable to impute.
   # (In such cases the imputation cycle cannot further converge, so 1 is efficient.)
-  # In the other cases, it is set to 5.
+  # In the other cases, it is set to 10.
   if(is.null(maxit)){
     if(length(variables_to_impute) <= 1){
       maxit <- 1
     }else{
-      maxit <- 5
+      maxit <- 10
     }
   }
 
@@ -419,12 +443,21 @@ How do you want to proceed: \n
   }
 
 
-  ########## Check of the rounding_formula
-  if(!is.list(rounding_formula)){
-    rounding_formula <- list_of_rounding_formulas_maker(data, default = rounding_formula)
+  # The rounding_formula shall be transformed into a list with vectors of covariates names mentioned in the
+  #rounding_formula
+  if(is.list(rounding_formula)){
+    rounding_covariates <- rounding_formula
+  }else{
+    rounding_covariates <- list()
+    for(tmpindex in names(data)){
+      rounding_covariates[[tmpindex]] <- rounding_formula
+    }
+    rm(tmpindex)
   }
-  for(i in names(rounding_formula)){
-    ge <- extract_varnames(model_formula = rounding_formula[[i]],
+
+
+  for(i in names(rounding_covariates)){
+    ge <- extract_varnames(model_formula = rounding_covariates[[i]],
                            constant_variables = constant_variables,
                            variable_names_in_data = variable_names_in_data,
                            data = data)
@@ -447,7 +480,7 @@ or [e]xiting the imputation?"))
 
 
     }
-    rounding_formula[[i]] <- ge$fixedeffects_varname
+    rounding_covariates[[i]] <- ge$fixedeffects_varname
   }
 
 
@@ -513,18 +546,20 @@ or [e]xiting the imputation?"))
   ########################################   START THE IMPUTATION #####################
 
   # get the variable types:
-  rounding_degrees_tmp <- rounding_degrees
-  types <- array(dim = ncol(my_data))
-
-  for(j in 1:ncol(my_data)){
-    if(is.list(rounding_degrees)){
-      rounding_degrees_tmp <- rounding_degrees[[colnames(my_data)[j]]]
-    }
-     types[j] <- get_type(my_data[, j], rounding_degrees = rounding_degrees_tmp)
-  }
+  types <- unlist(list_of_types_maker(my_data, spike = spike, rounding_degrees = rounding_degrees))
 
   categorical <- types == "categorical"
   if(any(categorical)){
+    more_than_10_levels <- colnames(my_data[, categorical, drop = FALSE])[
+      apply(my_data[, categorical, drop = FALSE], 2, function(x) nlevels(factor(x))) > 10]
+
+    if(length(more_than_10_levels) > 0 & !is.null(k_org)){
+      warning(paste("More than 10 categories found in ", paste(more_than_10_levels, collapse = ", "),
+                    "\n consider setting k = 10.
+                    \n Or if only insignificant categories should be dropped during imputation:
+                    \n set alpha = 0.2 (for example).", sep = ""))
+    }
+
     for(catcount in 1:sum(categorical)){
       my_data[, names(my_data)[categorical][catcount]] <-
         as.factor(my_data[, names(my_data)[categorical][catcount]])
@@ -532,8 +567,8 @@ or [e]xiting the imputation?"))
     }
   }
 
-
-  NA_locator <- is.na(my_data)
+  data_prepared <- my_data
+  NA_locator <- is.na(data_prepared)
   ####################   Preparation for setting up a mids-object
   alldata <- list()#this object saves the M datasets.
   my_chainMean <- array(dim = c(length(variables_to_impute), maxit, M))
@@ -550,10 +585,10 @@ or [e]xiting the imputation?"))
   tickts_drawn <- 0
 
   if(is.null(list_of_types)){
-    tmp_list_of_types <- list_of_types_maker(my_data, rounding_degrees = rounding_degrees)
+    tmp_list_of_types <- list_of_types_maker(my_data, spike = spike, rounding_degrees = rounding_degrees)
     #Note: here it can happen that a variable in the original_data is assumed to be
     #semicont, but after the first imputation it is considered to be continuous.
-    #Therefore we update the list of types absed on the data_before
+    #Therefore the list of types is updated based on the data_before
     #and not on the original data.
   }else{
     tmp_list_of_types <- list_of_types
@@ -567,7 +602,6 @@ or [e]xiting the imputation?"))
 
     for(l1 in 1:maxit){
 
-
       #do the imputation cycle
       tmp <- imputationcycle(data_before = my_data,
                              original_data = data,
@@ -577,19 +611,20 @@ or [e]xiting the imputation?"))
                              list_of_types = tmp_list_of_types,
                              nitt = nitt,
                              burnin = burnin,
-                             thin = thin,
+                             thin = 1,
                              pvalue = pvalue,
                              mn = mn,
-                             heap = heap,
+                             k = k,
+                             spike = spike,
                              rounding_degrees = rounding_degrees,
-                             rounding_formula = rounding_formula)
+                             rounding_covariates = rounding_covariates)
 
       my_data <- tmp$data_after
       gibbs[[paste("imputation", i, sep = "")]][[paste("cycle", l1, sep = "")]] <-
         tmp$chains
 
       if(is.null(list_of_types)){
-        tmp_list_of_types <- list_of_types_maker(data, rounding_degrees = rounding_degrees)
+        tmp_list_of_types <- list_of_types_maker(data, spike = spike, rounding_degrees = rounding_degrees)
         #Note: here it can happen that a variable in the original_data is assumed to be
         #semicont, but after the first imputation it is considered to be continuous.
         #Therefore we update the list of types absed on the data_before
@@ -605,8 +640,7 @@ or [e]xiting the imputation?"))
       #evaluate the imputed variables
       for(l2 in variables_to_impute){
 
-        to_evaluate <- my_data[is.na(data[, l2]), l2, drop = TRUE]
-        #for some variable types these imputed data coonet be directly evaluated by taking the mean.
+        #for some variable types these imputed data cannot be directly evaluated by taking the mean.
         # For example categorical variables (including binary variables which might not be 0-1 coded but
         # for example "m"-"f".)
 
@@ -616,21 +650,31 @@ or [e]xiting the imputation?"))
         #Therefore the types of the variables are needed
 
         if(is.null(list_of_types)){
-
-          if(is.list(rounding_degrees)){
-            rounding_degrees_tmp <- rounding_degrees[[l2]]
-          }else{
-            rounding_degrees_tmp <- rounding_degrees
-          }
-          tmp_type <- get_type(data[, l2], rounding_degrees = rounding_degrees_tmp)
+          tmp_type <- list_of_types_maker(data[, l2, drop = FALSE],
+                                          spike = spike, rounding_degrees = rounding_degrees)[[1]]
         }else{
           tmp_type <- list_of_types[[l2]]
         }
 
+        to_evaluate <- my_data[is.na(data[, l2]), l2, drop = TRUE]
 
         if(tmp_type == "roundedcont"){
+          #For rounded continuous variables three types of observations can be evaluated:
+          #1. the observations roundable by a factor from the rounding degrees.
+          #(Note: rounding to the nearest integer is counted as well, as those observations are imputed too)
+          #2. Interval observations.
+          #3. Missing observations.
+          if(is.list(rounding_degrees)){
+            rounding_degrees_tmp <- rounding_degrees[[l2]]
+          }else{
+            rounding_degrees_tmp <- suggest_rounding_degrees(data[, l2])
+          }
+          if(is.null(rounding_degrees_tmp)){
+            rounding_degrees_tmp <- c(1, 10, 100, 1000)
+          }
+
           eval_index <-
-            apply(outer(decompose_interval(interval = data[, l2])[, "precise"], setdiff(rounding_degrees_tmp, 1), '%%') == 0, 1, any) |
+            apply(outer(decompose_interval(interval = data[, l2])[, "precise"], rounding_degrees_tmp, '%%') == 0, 1, any) |
             !is.na(decompose_interval(interval = data[, l2])[, "lower_imprecise"]) | is.na(data[, l2])
 
           to_evaluate <- my_data[eval_index, l2, drop = TRUE]
@@ -646,12 +690,16 @@ or [e]xiting the imputation?"))
            tmp_type == "categorical" | tmp_type == "ordered_categorical"){
           eval_index <- is.na(data[, l2])
           to_evaluate <- as.numeric(as.factor(my_data[, l2]))[eval_index]
+        }
 
+        if(tmp_type == "intercept"){
+          to_evaluate <- 1
         }
 
         my_chainMean[l2, l1, i] <- mean(to_evaluate)
         my_chainVar[l2, l1, i] <- stats::var(to_evaluate)
       }
+
     }
 
 
@@ -670,27 +718,26 @@ or [e]xiting the imputation?"))
     alldata[[i]] <- data
     variables_to_export <- colnames(my_data) %in% colnames(data)
     alldata[[i]][, variables_to_export] <- my_data[, variables_to_export, drop = FALSE]
+
+    # Restore the original data, so that the next imputation round with its maxit cycles
+    # are independent of the current runs.
+    my_data <- data_prepared
+
   }
 
 
   # -------------- SET UP MIDS OBJECT  --------------
 
   imp_hmi <- stats::setNames(vector("list", ncol(data)), colnames(data))
-  method <- unlist(list_of_types_maker(data))
+  method <- unlist(list_of_types_maker(data, spike = spike, rounding_degrees = rounding_degrees))
   names(method) <- colnames(data)
 
   for(l2 in variables_to_impute){
 
-    ##get the number of missing values in each incomplete variable
-    # interval data are treated as missing
-    if(is.list(rounding_degrees)){
-      rounding_degrees_tmp <- rounding_degrees[[l2]]
-    }else{
-      rounding_degrees_tmp <- rounding_degrees
-    }
+    tmp_type <- list_of_types_maker(data[, l2, drop = FALSE],
+                 spike = spike, rounding_degrees = rounding_degrees)[[1]]
 
-    if(get_type(data[, l2], rounding_degrees = rounding_degrees_tmp) == "interval" |
-       get_type(data[, l2], rounding_degrees = rounding_degrees_tmp) == "roundedcont"){
+    if(tmp_type == "interval" |tmp_type == "roundedcont"){
       data[, l2] <- NA
     }
     n_mis_j <- sum(is.na(data[, l2]))
@@ -742,16 +789,24 @@ or [e]xiting the imputation?"))
 
   # If the user wants to use mice as a pooling function, and not hmi_pool,
   # we conduct the pooling right away using the model_formula given by the user
-  # ! Note: here we add something not found in mids-objects from mice !
-  tmp <- get_type(data[,fe$target_varname, drop = FALSE], rounding_degrees = rounding_degrees[[fe$target_varname]])
-  if(pool_with_mice & !is.null(model_formula_org) &
-     (tmp == "cont" | tmp == "semicont" | tmp == "roundedcont" | tmp == "interval" | tmp == "count")){
+  # ! Note: here something not found in mids-objects from mice is added!
+  tmp_type <- list_of_types_maker(data[, fe$target_varname, drop = FALSE], spike = spike,
+                                  rounding_degrees = rounding_degrees)[[1]]
+
+  if(is.null(tmp_type)){
+    tmp_type <- "unknown"
+  }
+
+  #For cautious pooling, include:
+  #& (tmp_type == "cont" | tmp_type == "semicont" | tmp_type == "roundedcont" |
+  #    tmp_type == "interval" | tmp_type == "count")
+  if(pool_with_mice & !is.null(model_formula_org)){
     if(fe$clID_varname == ""){ # if no cluster ID was found, run a single level model
 
       # mice::pool cannot deal with missing coefficients, so pool is only called,
       #if noch coefficitent is NA.
       mira <- with(data = midsobj,
-                   expr = stats::lm(formula = formula(format(model_formula_org))))
+                   expr = stats::glm(formula = formula(format(model_formula_org)), family = family))
 
       if(!any(unlist(lapply(mira$analyses, function(x) any(is.na(stats::coefficients(x))))))){
         midsobj$pooling <- mice::pool(mira)
@@ -760,14 +815,15 @@ or [e]xiting the imputation?"))
 
     }else{ # otherwise a multilevel model
       midsobj$pooling <- mice::pool(with(data = midsobj,
-                                         expr = lme4::lmer(formula = format(model_formula_org))))
+                                         expr = lme4::glmer(formula = format(model_formula_org),
+                                                            family = family)))
 
     }
   }
 
   #In addition to classical mids objects we add the chains of the gibbs-samplings from MCMCglmm.
   midsobj$gibbs <- gibbs
-  midsobj$gibbs_para <- c(nitt, burnin, thin)
+  midsobj$gibbs_para <- c(nitt, burnin, 1) #The last parameter is the default thinning parameter.
 
 
   return(midsobj)
