@@ -34,9 +34,6 @@ library("hmi")
 set.seed(1)
 result <- hmi(ex)
 
-## ----eval = FALSE, message = FALSE, cache = TRUE, fig.width = 8----------
-#  plot(result, layout = c(2, 1))
-
 ## ----eval = TRUE, message = FALSE, cache = TRUE--------------------------
 head(complete(result, 1))
 
@@ -54,12 +51,11 @@ lmer(formula = Reaction~1+Days+(1+Days|Subject), data = ex, na.action = na.omit)
 
 ## ----eval = TRUE, echo = FALSE, results = "hide", message = FALSE, cache = TRUE----
 library("hmi")
-ex
 set.seed(1)
 result_multi <- hmi(data = ex, model_formula = Reaction ~ 1 + Days + (1 + Days | Subject), maxit = 1)
 
 ## ----eval = TRUE, echo = TRUE, message = FALSE, cache = TRUE-------------
-pool(with(data = result_multi, expr = lmer(Reaction ~ 1 + Days + (1 + Days | Subject))))
+result_multi$pooling
 
 ## ----eval = FALSE, echo = TRUE, message = FALSE, cache = TRUE------------
 #   my_analysis <- function(complete_data){
@@ -74,15 +70,15 @@ pool(with(data = result_multi, expr = lmer(Reaction ~ 1 + Days + (1 + Days | Sub
 #    # the random slopes variance and their covariance
 #    my_model <- lmer(Reaction ~ 1 + Days + (1 + Days | Subject), data = complete_data)
 #  
-#    parameters_of_interest[[1]] <- fixef(my_model)[1]
-#    parameters_of_interest[[2]] <- fixef(my_model)[2]
-#    parameters_of_interest[[3]] <- VarCorr(my_model)[[1]][1, 1]
-#    parameters_of_interest[[4]] <- VarCorr(my_model)[[1]][1, 2]
-#    parameters_of_interest[[5]] <- VarCorr(my_model)[[1]][2, 2]
-#    names(parameters_of_interest) <- c("beta_0", "beta_1", "sigma0", "sigma01", "sigma1")
+#     parameters_of_interest[[1]] <- fixef(my_model)
+#     parameters_of_interest[[2]] <- lme4::VarCorr(my_model)[[1]][,]
+#     ret <- unlist(parameters_of_interest)# This line is essential if the elements of interest
+#     #should be labeled in the following line.
+#     names(ret) <-
+#       c("beta0", "beta1", "sigma0", "sigma01", "sigma10", "sigma1")
 #  
 #    # ---- do not change this function below this line.
-#    return(parameters_of_interest)
+#    return(ret)
 #   }
 #  
 #  hmi_pool(mids = result_multi, analysis_function = my_analysis)
@@ -101,15 +97,15 @@ library("hmi")
   # the random slopes variance and their covariance
   my_model <- lmer(Reaction ~ 1 + Days + (1 + Days | Subject), data = complete_data)
 
-  parameters_of_interest[[1]] <- fixef(my_model)[1]
-  parameters_of_interest[[2]] <- fixef(my_model)[2]
-  parameters_of_interest[[3]] <- VarCorr(my_model)[[1]][1, 1]
-  parameters_of_interest[[4]] <- VarCorr(my_model)[[1]][1, 2]
-  parameters_of_interest[[5]] <- VarCorr(my_model)[[1]][2, 2]
-  names(parameters_of_interest) <- c("beta_0", "beta_1", "sigma0", "sigma01", "sigma1")
 
-  # ---- do change this function below this line.
-  return(parameters_of_interest)
+   parameters_of_interest[[1]] <- fixef(my_model)
+   parameters_of_interest[[2]] <- lme4::VarCorr(my_model)[[1]][,]
+   ret <- unlist(parameters_of_interest)# This line is essential if the elements of interest
+   #should be labeled in the following line.
+   names(ret) <-
+     c("beta0", "beta1", "sigma0", "sigma01", "sigma10", "sigma1")
+
+   return(ret)
  }
 
 hmi_pool(mids = result_multi, analysis_function = my_analysis)
